#include <Message.h>
#include <InterfaceDefs.h>
#include <File.h>

#include "FancyFilter.h"
#include "FancySlave.h"
#include "Opcodes.h"

BInputServerFilter *instantiate_input_filter()
{
	return new FancyFilter;
}

FancyFilter::FancyFilter()
{
	slave = new FancySlave;
	slave->Run();
}

FancyFilter::~FancyFilter()
{
	slave->PostMessage(B_QUIT_REQUESTED);
}

filter_result FancyFilter::Filter(BMessage *msg, BList *outlist)
{
	switch (msg->what) {
	case B_KEY_DOWN:
	case B_UNMAPPED_KEY_DOWN:
		break;
	default:
		return B_DISPATCH_MESSAGE;
	}

	int32 key;
	if (msg->FindInt32("key", &key) != B_OK)
		return B_DISPATCH_MESSAGE;

	if (key < 0x100000)
		// special keys are >= 0x100000
		return B_DISPATCH_MESSAGE;

	uint32 modifiers = 0;
	msg->FindInt32("modifiers", (int32 *) &modifiers);

	if (modifiers & B_COMMAND_KEY)
		return B_DISPATCH_MESSAGE;
	bool shift = modifiers & B_SHIFT_KEY;
	bool control = modifiers & B_CONTROL_KEY;
	bool option = modifiers & B_OPTION_KEY;

	uint32 what = 0;
	if (!shift && !control && !option)
		switch (key) {
		case key::Mute:		what = opcode::Mute;		break;
		case key::VolumeUp:	what = opcode::VolumeUp;		break;
		case key::VolumeDown:	what = opcode::VolumeDown;	break;
		case key::Stop:		what = opcode::Stop;		break;
		case key::Eject:		what = opcode::Eject;		break;
		case key::PlayPause:	what = opcode::PlayPause;		break;
		case key::NextTrack:	what = opcode::NextTrack;	break;
		case key::PrevTrack:	what = opcode::PrevTrack;	break;
		default:			return B_DISPATCH_MESSAGE;
		}
	else if (shift && !control && !option)
		switch (key) {
		case key::Stop:		what = opcode::Eject;		break;
		case key::PlayPause:	what = opcode::PlaySelection;	break;
		case key::NextTrack:	what = opcode::LastTrack;		break;
		case key::PrevTrack:	what = opcode::FirstTrack;		break;
		default:			return B_DISPATCH_MESSAGE;
		}
	else if (!shift && control && !option)
		switch (key) {
		case key::PlayPause:	what = opcode::AddSelection;	break;
		default:			return B_DISPATCH_MESSAGE;
		}
	else if (!shift && !control && option)
		switch (key) {
		case key::PlayPause:	what = opcode::Shuffle;		break;
		default:			return B_DISPATCH_MESSAGE;
		}
	else
		return B_DISPATCH_MESSAGE;

	slave->PostMessage(what);
	return B_SKIP_MESSAGE;
}
