#ifndef FANCY_PLAYER_H
#define FANCY_PLAYER_H

#include <Entry.h>
#include <Locker.h>
#include <Looper.h>
#include <MediaDefs.h>

#include <vector>

#include "FancyPlayerInterface.h"

class FancyPlayer : public BLooper {
public:
	FancyPlayer(const char *name);
	~FancyPlayer();

	void MessageReceived(BMessage *msg);

private:
	static void FillBuffer(void *cookie, void *buffer, size_t bufferSize, const media_raw_audio_format &format);
	void FillBuffer(void *buffer, size_t bufferSize, const media_raw_audio_format &format);

	FPInterface::Status GetStatus() const;

	status_t Play();
	status_t Pause();
	status_t Eject();
	status_t Load(const entry_ref &);
	status_t Seek(bigtime_t &, bool sloppy);

	status_t RegisterCallback(BMessenger &);
	status_t RemoveCallback(const BMessenger &);

	bool HasFile() const { if (!fFileData) return false; return fFileData->isValid; }

	void Notify(uint32 flags, BMessenger *single_messenger = 0);

	enum { EndOfTrackReached = 'eotr' };

	enum { 	QuitBit = 1,
			EndOfTrackBit = 2,
			StatusBit = 4 };

	class StatusMonitor {
	public:
		StatusMonitor(FancyPlayer *which);

		void Mark();
		uint32 Changed() const;

		FancyPlayer *which;
		entry_ref ref;
		FPInterface::Status status;
	};
	friend class StatusMonitor;

	class AutoNotify {
		friend class FancyPlayer;

		AutoNotify(FancyPlayer *which);
		~AutoNotify();

		FancyPlayer *which;
		StatusMonitor mon;
	};
	int fAutoNotifyStack;
	friend class AutoNotify;

	typedef vector<BMessenger> notifier_list_t;
	notifier_list_t fNotifierList;

	class FileData {
	public:
		FileData(const entry_ref &);
		~FileData();

		bool isValid;
		entry_ref ref;
		BMediaFile *file;
		BMediaTrack *track;
		media_format format;

	private:
		FileData(const FileData &);
		const FileData &operator=(const FileData &);
	};

	bool fPaused;
	FileData *fFileData;
	BSoundPlayer *fPlayer;

	BLocker fAudioLock;
	// begin - locked by fAudioLock
		int8 fDefaultData;
		uint32 fBufferSize;
		uint32 fFrameSize;
		bigtime_t fPerfTime;
		bigtime_t fTrackTime;
		bigtime_t fFinalTime;
		float fFrameRate;
	// end - locked by fAudioLock
};

bool IsAudioFile(const entry_ref &);
	// Sniff file for audio tracks

#endif
