#ifndef FANCY_PLAYER_INTERFACE_H
#define FANCY_PLAYER_INTERFACE_H

namespace FPInterface {

	enum Status {
		Empty		= 'emty',
		Paused	= 'paus',
		Playing	= 'play'
	};

	enum Commands {
		Eject		= 'ejct',
		Pause		= 'paus',
		Play		= 'play',

		Seek		= 'seek',

		GetStatus		= 'gstt',
		GetFileInfo		= 'gfif',
		GetTime		= 'gtim',
		Load			= 'load',

		RegisterCallback	= 'rgcb',
		RemoveCallback	= 'dlcb',

		GetCallbacks		= 'cgbs'
	};

	enum Notifications {
		StatusNotify		= 'stan',
		EndOfFileNotify	= 'eofn',
		QuitNotify		= 'qitn'
	};

}

///////////////////////////////////////////////////
// Description of messages
/*

Eject, Pause, Play:
	reply:
		what = B_OK | error
		uint32:"change" = NotificationFlags indicating effect

	note:
		Pause does not "UnPause".

Seek:
	bigtime_t:"time" = seek to this time
	[bool:"sloppy" = don't need to be too precise] default = true

	reply:
		what = B_OK | B_ERROR
		[bigtime_t:"time" = position after seek]

GetStatus:
	reply:
		what = Empty | Paused | Playing

GetFileInfo:
	reply:
		what = B_OK | error
		[entry_ref:"ref" = current file]
		[bigtime_t:"length" = length of track]

GetTime:
	reply:
		what = B_OK | error
		[bigtime_t:"time" = current time in track]

Load:
	entry_ref:"ref" = file to load
	[bool:"play" = start playing?] default = false

	reply:
		what = B_OK | error

	note:
		A failed load will not alter the state of the player.

RegisterCallback:
	BMessenger:"messenger" = messenger that will receive notivications

	reply:
		what = B_OK | error

	note:
		Immediately after a new callback is registered it will be
		sent the current status and file.

RemoveCallback:
	BMessenger:"messenger" = messenger to remove from callback list

	reply:
		what = B_OK | error

GetCallbacks:
	reply:
		what = B_OK | error
		[BMessenger:"messengers" = the registered callbacks]

	note:
		Before filling the "messengers" array, all messengers will be
		validated. Invalid messengers will be removed.

StatusNotify: (notification)
	uint32:"status" = current status
	[entry_ref:"ref" = current file]
	[bigtime_t:"length" = length of track]
	bool:"ended" = was this message sent because the track ended?

QuitNotify: (notification)
	empty

EndOfFileNotify:
	entry_ref:"ref" = file that ended
	bigtime_t:"length" = true length of track

	note:
		This message will be send immediately after the StatusNotify
		indicating that playback stopped.
		"length" contains the true length of the track, previous
		legths have been approximations.

*/

#endif
