#include "FancyDebug.h"

#include <Entry.h>
#include <Message.h>
#include <StringView.h>
#include <Window.h>

#include "FancyPlayerInterface.h"
#include "TimedWindow.h"
#include "PlayModeView.h"
#include "PlayStatusView.h"

PlayStatusView::PlayStatusView()
	:	BView(BRect(0,0,100,20), "PlayStatusView", B_FOLLOW_NONE, B_WILL_DRAW)
{	ENTRY

	SetViewColor(0,0,0);
	fPlayModeView = new PlayModeView(BRect(4,4,14,14));
	AddChild(fPlayModeView);

	fTitleView = new BStringView(BRect(20,2,50,30), "TitleView", "Should not see this");
	BFont f(be_plain_font);
	f.SetSize(15);
	f.SetFace(B_BOLD_FACE);
	fTitleView->SetFont(&f);
	fTitleView->SetViewColor(0,0,0);
	fTitleView->SetLowColor(0,0,0);
	fTitleView->SetHighColor(50, 190, 255);

	AddChild(fTitleView);

	ResizeToPreferred();
}

void PlayStatusView::ResizeToPreferred()
{
	fTitleView->ResizeToPreferred();
	fTitleView->MoveTo(fPlayModeView->Frame().right + 2, 0);

	ResizeTo(fTitleView->Frame().Width()+fPlayModeView->Frame().Width()+5, fTitleView->Frame().Height());
}

void PlayStatusView::MessageReceived(BMessage *msg)
{	ENTRY

	switch (msg->what) {
	case BogusStatus:
		{
			uint32 stat;
			if (msg->FindInt32("status", reinterpret_cast<int32 *>(&stat)) == B_OK) {
				PlayModeView::PlayMode pm;
				switch (stat) {
				case FPInterface::Playing: pm = PlayModeView::Playing; break;
				case FPInterface::Paused: pm = PlayModeView::Paused; break;
				case FPInterface::Empty: pm = PlayModeView::Stopped; break;
				default: return;
				}
				fPlayModeView->SetMode(pm);
			}

			entry_ref ref;
			if (msg->FindRef("ref", &ref) == B_OK)
				fTitleView->SetText(GetTitle(ref).c_str());

			static_cast<TimedWindow *>(Window())->Place();
			Window()->Show();
		}
		break;
	case FPInterface::StatusNotify:
		{
			bool ended;
			if (msg->FindBool("ended", &ended) == B_OK)
				if (ended)
					return;

			uint32 stat;
			if (msg->FindInt32("status", reinterpret_cast<int32 *>(&stat)) == B_OK) {
				PlayModeView::PlayMode pm;
				switch (stat) {
				case FPInterface::Playing: pm = PlayModeView::Playing; break;
				case FPInterface::Paused: pm = PlayModeView::Paused; break;
				default: return;
				}
				fPlayModeView->SetMode(pm);
			}

			entry_ref ref;
			if (msg->FindRef("ref", &ref) == B_OK)
				fTitleView->SetText(GetTitle(ref).c_str());

			static_cast<TimedWindow *>(Window())->Place();
			Window()->Show();
		}
		break;

	default:
		BView::MessageReceived(msg);
	}
}

string PlayStatusView::GetTitle(entry_ref &ref)
{
	BEntry e(&ref, true);
	if (e.InitCheck() != B_OK)
		return "Error";

	char name[B_FILE_NAME_LENGTH];
	name[0] = '\0';

	BNode node(&e);
	if (node.InitCheck() == B_OK) {
		char title[1024];
		char artist[1024];
		ssize_t tlen = 0, alen = 0;

		if ((tlen = node.ReadAttr("Audio:Title", B_STRING_TYPE, 0, title, 1023)) > 0) {
			title[tlen] = '\0';
			if (tlen <= 2 && (title[0] == '\0' || title[0] == ' '))
				tlen = 0;
		} else tlen = 0;

		if (tlen) {
			if ((alen = node.ReadAttr("Audio:Artist", B_STRING_TYPE, 0, artist, 1023)) > 0) {
				artist[alen] = '\0';
				if (alen <= 2 && (artist[0] == '\0' || artist[0] == ' '))
					alen = 0;
			} else alen = 0;
		}

		if (tlen && alen) {
			strcpy(name, artist);
			strcat(name, " - ");
			strcat(name, title);
		} else if (tlen) {
			strcpy(name, title);
		}
	}

	if (name[0] == '\0')
		e.GetName(name);

	return name;
}
